@@ -4,6 +4,7 @@ require 'net/https'
 require 'uri'
 require 'zlib'
 require 'multi_xml'
+require 'mime/types'
 require 'json'
 require 'csv'
 
@@ -13,6 +14,9 @@ require 'httparty/net_digest_auth'
 require 'httparty/version'
 require 'httparty/connection_adapter'
 require 'httparty/logger/logger'
+require 'httparty/request/body'
+require 'httparty/response_fragment'
+require 'httparty/text_encoder'
 
 # @see HTTParty::ClassMethods
 module HTTParty
@@ -40,6 +44,7 @@ module HTTParty
   # [:+local_port:] Local port to bind to before connecting.
   # [:+body_stream:] Allow streaming to a REST server to specify a body_stream.
   # [:+stream_body:] Allow for streaming large files without loading them into memory.
+  # [:+multipart:] Force content-type to be multipart
   #
   # There are also another set of options with names corresponding to various class methods. The methods in question are those that let you set a class-wide default, and the options override the defaults on a request-by-request basis. Those options are:
   # * :+base_uri+: see HTTParty::ClassMethods.base_uri.
@@ -168,9 +173,9 @@ module HTTParty
     #     include HTTParty
     #     default_timeout 10
     #   end
-    def default_timeout(t)
-      raise ArgumentError, 'Timeout must be an integer or float' unless t && (t.is_a?(Integer) || t.is_a?(Float))
-      default_options[:timeout] = t
+    def default_timeout(value)
+      validate_timeout_argument(__method__, value)
+      default_options[:timeout] = value
     end
 
     # Allows setting a default open_timeout for all HTTP calls in seconds
@@ -179,9 +184,9 @@ module HTTParty
     #     include HTTParty
     #     open_timeout 10
     #   end
-    def open_timeout(t)
-      raise ArgumentError, 'open_timeout must be an integer or float' unless t && (t.is_a?(Integer) || t.is_a?(Float))
-      default_options[:open_timeout] = t
+    def open_timeout(value)
+      validate_timeout_argument(__method__, value)
+      default_options[:open_timeout] = value
     end
 
     # Allows setting a default read_timeout for all HTTP calls in seconds
@@ -190,11 +195,24 @@ module HTTParty
     #     include HTTParty
     #     read_timeout 10
     #   end
-    def read_timeout(t)
-      raise ArgumentError, 'read_timeout must be an integer or float' unless t && (t.is_a?(Integer) || t.is_a?(Float))
-      default_options[:read_timeout] = t
+    def read_timeout(value)
+      validate_timeout_argument(__method__, value)
+      default_options[:read_timeout] = value
     end
 
+    # Allows setting a default write_timeout for all HTTP calls in seconds
+    # Supported by Ruby > 2.6.0
+    #
+    #   class Foo
+    #     include HTTParty
+    #     write_timeout 10
+    #   end
+    def write_timeout(value)
+      validate_timeout_argument(__method__, value)
+      default_options[:write_timeout] = value
+    end
+
+
     # Set an output stream for debugging, defaults to $stderr.
     # The output stream is passed on to Net::HTTP#set_debug_output.
     #
@@ -546,10 +564,22 @@ module HTTParty
       perform_request Net::HTTP::Mkcol, path, options, &block
     end
 
+    def lock(path, options = {}, &block)
+      perform_request Net::HTTP::Lock, path, options, &block
+    end
+
+    def unlock(path, options = {}, &block)
+      perform_request Net::HTTP::Unlock, path, options, &block
+    end
+
     attr_reader :default_options
 
     private
 
+    def validate_timeout_argument(timeout_type, value)
+      raise ArgumentError, "#{ timeout_type } must be an integer or float" unless value && (value.is_a?(Integer) || value.is_a?(Float))
+    end
+
     def ensure_method_maintained_across_redirects(options)
       unless options.key?(:maintain_method_across_redirects)
         options[:maintain_method_across_redirects] = true
@@ -564,8 +594,19 @@ module HTTParty
     end
 
     def process_headers(options)
-      if options[:headers] && headers.any?
-        options[:headers] = headers.merge(options[:headers])
+      if options[:headers]
+        if headers.any?
+          options[:headers] = headers.merge(options[:headers])
+        end
+
+        options[:headers] = Utils.stringify_keys(process_dynamic_headers(options[:headers]))
+      end
+    end
+
+    def process_dynamic_headers(headers)
+      headers.each_with_object({}) do |header, processed_headers|
+        key, value = header
+        processed_headers[key] = value.respond_to?(:call) ? value.call : value
       end
     end
 
@@ -577,7 +618,8 @@ module HTTParty
 
     def validate_format
       if format && parser.respond_to?(:supports_format?) && !parser.supports_format?(format)
-        raise UnsupportedFormat, "'#{format.inspect}' Must be one of: #{parser.supported_formats.map(&:to_s).sort.join(', ')}"
+        supported_format_names = parser.supported_formats.map(&:to_s).sort.join(', ')
+        raise UnsupportedFormat, "'#{format.inspect}' Must be one of: #{supported_format_names}"
       end
     end
   end
@@ -635,6 +677,7 @@ module HTTParty
 end
 
 require 'httparty/hash_conversions'
+require 'httparty/utils'
 require 'httparty/exceptions'
 require 'httparty/parser'
 require 'httparty/request'
