@@ -4,6 +4,12 @@ module HTTParty
       string.gsub(/([A-Z]+)([A-Z][a-z])/, '\1_\2').gsub(/([a-z])([A-Z])/, '\1_\2').downcase
     end
 
+    def self._load(data)
+      req, resp, parsed_resp, resp_body = Marshal.load(data)
+
+      new(req, resp, -> { parsed_resp }, body: resp_body)
+    end
+
     attr_reader :request, :response, :body, :headers
 
     def initialize(request, response, parsed_block, options = {})
@@ -14,7 +20,11 @@ module HTTParty
       @headers      = Headers.new(response.to_hash)
 
       if request.options[:logger]
-        logger = ::HTTParty::Logger.build(request.options[:logger], request.options[:log_level], request.options[:log_format])
+        logger = ::HTTParty::Logger.build(
+          request.options[:logger],
+          request.options[:log_level],
+          request.options[:log_format]
+        )
         logger.format(request, self)
       end
 
@@ -29,6 +39,10 @@ module HTTParty
       response.code.to_i
     end
 
+    def http_version
+      response.http_version
+    end
+
     def tap
       yield self
       self
@@ -55,14 +69,23 @@ module HTTParty
       alias_method :multiple_choice?, :multiple_choices?
     end
 
+    # Support old status codes method from pre 2.6.0 era.
+    if ::RUBY_VERSION >= "2.6.0" && ::RUBY_PLATFORM != "java"
+      alias_method :gateway_time_out?,                :gateway_timeout?
+      alias_method :request_entity_too_large?,        :payload_too_large?
+      alias_method :request_time_out?,                :request_timeout?
+      alias_method :request_uri_too_long?,            :uri_too_long?
+      alias_method :requested_range_not_satisfiable?, :range_not_satisfiable?
+    end
+
     def nil?
       response.nil? || response.body.nil? || response.body.empty?
     end
 
-    def to_s      
+    def to_s
       if !response.nil? && !response.body.nil? && response.body.respond_to?(:to_s)
         response.body.to_s
-      else 
+      else
         inspect
       end
     end
@@ -80,7 +103,7 @@ module HTTParty
         parsed_response.display(port)
       elsif !response.nil? && !response.body.nil? && response.body.respond_to?(:display)
         response.body.display(port)
-      else 
+      else
         port.write(inspect)
       end
     end
@@ -89,7 +112,11 @@ module HTTParty
       return true if super
       parsed_response.respond_to?(name) || response.respond_to?(name)
     end
-    
+
+    def _dump(_level)
+      Marshal.dump([request, response, parsed_response, body])
+    end
+
     protected
 
     def method_missing(name, *args, &block)
