@@ -1,4 +1,4 @@
-require File.expand_path(File.join(File.dirname(__FILE__), '..', 'spec_helper'))
+require 'spec_helper'
 
 RSpec.describe HTTParty::Request do
   before do
@@ -73,7 +73,7 @@ RSpec.describe HTTParty::Request do
       expect(request.parser).to eq(my_parser)
     end
 
-    it "sets connection_adapter to HTTPParty::ConnectionAdapter" do
+    it "sets connection_adapter to HTTParty::ConnectionAdapter" do
       request = HTTParty::Request.new(Net::HTTP::Get, 'http://google.com')
       expect(request.connection_adapter).to eq(HTTParty::ConnectionAdapter)
     end
@@ -343,22 +343,35 @@ RSpec.describe HTTParty::Request do
       end
     end
 
-    context "when body is multipart" do
-      it "sets header Content-Type: multipart/form-data; boundary=" do
-        @request.options[:body] = {file: File.open(File::NULL, 'r')}
+    context "when mulipart" do
+      subject(:headers) do
         @request.send(:setup_raw_request)
         headers = @request.instance_variable_get(:@raw_request).each_header.to_a
-        headers = Hash[*headers.flatten]  # Ruby 2.0 doesn't have Array#to_h
-        expect(headers['content-type']).to match(%r{^multipart/form-data; boundary=---})
+        Hash[*headers.flatten]  # Ruby 2.0 doesn't have Array#to_h
       end
 
-      context "and header Content-Type is provided" do
-        it "overwrites the header to: multipart/form-data; boundary=" do
+      context "when body contains file" do
+        it "sets header Content-Type: multipart/form-data; boundary=" do
           @request.options[:body] = {file: File.open(File::NULL, 'r')}
-          @request.options[:headers] = {'Content-Type' => 'application/x-www-form-urlencoded'}
-          @request.send(:setup_raw_request)
-          headers = @request.instance_variable_get(:@raw_request).each_header.to_a
-          headers = Hash[*headers.flatten]  # Ruby 2.0 doesn't have Array#to_h
+
+          expect(headers['content-type']).to match(%r{^multipart/form-data; boundary=---})
+        end
+
+        context "and header Content-Type is provided" do
+          it "overwrites the header to: multipart/form-data; boundary=" do
+            @request.options[:body] = {file: File.open(File::NULL, 'r')}
+            @request.options[:headers] = {'Content-Type' => 'application/x-www-form-urlencoded'}
+
+            expect(headers['content-type']).to match(%r{^multipart/form-data; boundary=---})
+          end
+        end
+      end
+
+      context 'when mulipart option is provided' do
+        it "sets header Content-Type: multipart/form-data; boundary=" do
+          @request.options[:body] = { text: 'something' }
+          @request.options[:multipart] = true
+
           expect(headers['content-type']).to match(%r{^multipart/form-data; boundary=---})
         end
       end
@@ -492,50 +505,72 @@ RSpec.describe HTTParty::Request do
     end
 
     if "".respond_to?(:encoding)
+      context 'when body has ascii-8bit encoding' do
+        let(:response) { stub_response "Content".force_encoding('ascii-8bit') }
 
-      let(:response_charset) {
-        @request.send(:get_charset)
-      }
+        it "processes charset in content type properly" do
+          response.initialize_http_header("Content-Type" => "text/plain;charset = utf-8")
+          resp = @request.perform
 
-      it "should process charset in content type properly" do
-        response = stub_response "Content".force_encoding('ascii-8bit')
-        response.initialize_http_header("Content-Type" => "text/plain;charset = utf-8")
-        resp = @request.perform
-        expect(response_charset).to_not be_empty
-        expect(resp.body.encoding).to eq(Encoding.find("UTF-8"))
-      end
+          expect(resp.body.encoding).to eq(Encoding.find("UTF-8"))
+        end
 
-      it "should process charset in content type properly if it has a different case" do
-        response = stub_response "Content".force_encoding('ascii-8bit')
-        response.initialize_http_header("Content-Type" => "text/plain;CHARSET = utf-8")
-        resp = @request.perform
-        expect(response_charset).to_not be_empty
-        expect(resp.body.encoding).to eq(Encoding.find("UTF-8"))
-      end
+        it "processes charset in content type properly if it has a different case" do
+          response.initialize_http_header("Content-Type" => "text/plain;CHARSET = utf-8")
+          resp = @request.perform
 
-      it "should process quoted charset in content type properly" do
-        response = stub_response "Content".force_encoding('ascii-8bit')
-        response.initialize_http_header("Content-Type" => "text/plain;charset = \"utf-8\"")
-        resp = @request.perform
-        expect(response_charset).to_not be_empty
-        expect(resp.body.encoding).to eq(Encoding.find("UTF-8"))
+          expect(resp.body.encoding).to eq(Encoding.find("UTF-8"))
+        end
+
+        it "processes quoted charset in content type properly" do
+          response.initialize_http_header("Content-Type" => "text/plain;charset = \"utf-8\"")
+          resp = @request.perform
+
+          expect(resp.body.encoding).to eq(Encoding.find("UTF-8"))
+        end
+
+        context 'when stubed body is frozen' do
+          let(:response) do
+            stub_response "Content".force_encoding('ascii-8bit').freeze
+          end
+
+          it 'processes frozen body correctly' do
+            response.initialize_http_header("Content-Type" => "text/plain;charset = utf-8")
+            resp = @request.perform
+
+            expect(resp.body.encoding).to eq(Encoding.find("UTF-8"))
+          end
+        end
       end
 
       it "should process response with a nil body" do
         response = stub_response nil
         response.initialize_http_header("Content-Type" => "text/html;charset=UTF-8")
         resp = @request.perform
+
         expect(resp.body).to be_nil
       end
 
-      it "should process utf-16 charset with little endian bom correctly" do
-        @request.options[:assume_utf16_is_big_endian] = true
+      context 'when assume_utf16_is_big_endian is true' do
+        before { @request.options[:assume_utf16_is_big_endian] = true }
 
-        response = stub_response "\xFF\xFEC\x00o\x00n\x00t\x00e\x00n\x00t\x00"
-        response.initialize_http_header("Content-Type" => "text/plain;charset = utf-16")
-        resp = @request.perform
-        expect(response_charset).to_not be_empty
-        expect(resp.body.encoding).to eq(Encoding.find("UTF-16LE"))
+        it "should process utf-16 charset with little endian bom correctly" do
+          response = stub_response "\xFF\xFEC\x00o\x00n\x00t\x00e\x00n\x00t\x00"
+
+          response.initialize_http_header("Content-Type" => "text/plain;charset = utf-16")
+          resp = @request.perform
+
+          expect(resp.body.encoding).to eq(Encoding.find("UTF-16LE"))
+        end
+
+        it 'processes stubbed frozen body correctly' do
+          response = stub_response "\xFF\xFEC\x00o\x00n\x00t\x00e\x00n\x00t\x00".freeze
+
+          response.initialize_http_header("Content-Type" => "text/plain;charset = utf-16")
+          resp = @request.perform
+
+          expect(resp.body.encoding).to eq(Encoding.find("UTF-16LE"))
+        end
       end
 
       it "should process utf-16 charset with big endian bom correctly" do
@@ -544,7 +579,7 @@ RSpec.describe HTTParty::Request do
         response = stub_response "\xFE\xFF\x00C\x00o\x00n\x00t\x00e\x00n\x00t"
         response.initialize_http_header("Content-Type" => "text/plain;charset = utf-16")
         resp = @request.perform
-        expect(response_charset).to_not be_empty
+
         expect(resp.body.encoding).to eq(Encoding.find("UTF-16BE"))
       end
 
@@ -554,7 +589,7 @@ RSpec.describe HTTParty::Request do
         response = stub_response "C\x00o\x00n\x00t\x00e\x00n\x00t\x00"
         response.initialize_http_header("Content-Type" => "text/plain;charset = utf-16")
         resp = @request.perform
-        expect(response_charset).to_not be_empty
+
         expect(resp.body.encoding).to eq(Encoding.find("UTF-16LE"))
       end
 
@@ -562,9 +597,9 @@ RSpec.describe HTTParty::Request do
         response = stub_response "Content"
         response.initialize_http_header("Content-Type" => "text/plain;charset = utf-lols")
         resp = @request.perform
-        expect(response_charset).to_not be_empty
+
         # This encoding does not exist, thus the string should not be encodd with it
-        expect(resp.body.encoding).to_not eq(response_charset)
+
         expect(resp.body).to eq("Content")
         expect(resp.body.encoding).to eq("Content".encoding)
       end
@@ -573,7 +608,7 @@ RSpec.describe HTTParty::Request do
         response = stub_response "Content"
         response.initialize_http_header("Content-Type" => "text/plain")
         resp = @request.perform
-        expect(response_charset).to be_nil
+
         expect(resp.body).to eq("Content")
         expect(resp.body.encoding).to eq("Content".encoding)
       end
@@ -788,6 +823,16 @@ RSpec.describe HTTParty::Request do
           expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
         end
 
+        it "should be handled by LOCK transparently" do
+          @request.http_method = Net::HTTP::Lock
+          expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
+        end
+
+        it "should be handled by UNLOCK transparently" do
+          @request.http_method = Net::HTTP::Unlock
+          expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
+        end
+
         it "should keep track of cookies between redirects" do
           @redirect['Set-Cookie'] = 'foo=bar; name=value; HTTPOnly'
           @request.perform
@@ -914,6 +959,16 @@ RSpec.describe HTTParty::Request do
         expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
       end
 
+      it "should be handled by LOCK transparently" do
+        @request.http_method = Net::HTTP::Lock
+        expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
+      end
+
+      it "should be handled by UNLOCK transparently" do
+        @request.http_method = Net::HTTP::Unlock
+        expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
+      end
+
       it "should keep track of cookies between redirects" do
         @redirect['Set-Cookie'] = 'foo=bar; name=value; HTTPOnly'
         @request.perform
@@ -1052,6 +1107,16 @@ RSpec.describe HTTParty::Request do
       expect(@request.perform.code).to eq(304)
     end
 
+    it "should be handled by LOCK transparently" do
+      @request.http_method = Net::HTTP::Lock
+      expect(@request.perform.code).to eq(304)
+    end
+
+    it "should be handled by UNLOCK transparently" do
+      @request.http_method = Net::HTTP::Unlock
+      expect(@request.perform.code).to eq(304)
+    end
+
     it 'should not log the redirection' do
       logger_double = double
       expect(logger_double).to receive(:info).once
@@ -1123,6 +1188,18 @@ RSpec.describe HTTParty::Request do
           expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
         end
 
+        it "should be handled by LOCK transparently" do
+          @request.http_method = Net::HTTP::Lock
+          expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
+        end
+
+        it "should be handled by UNLOCK transparently" do
+          @request.http_method = Net::HTTP::Unlock
+          expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
+        end
+
+
+
         it "should keep track of cookies between redirects" do
           @redirect['Set-Cookie'] = 'foo=bar; name=value; HTTPOnly'
           @request.perform
