@@ -1,4 +1,4 @@
-require File.expand_path(File.join(File.dirname(__FILE__), 'spec_helper'))
+require_relative 'spec_helper'
 
 RSpec.describe HTTParty do
   before(:each) do
@@ -144,34 +144,44 @@ RSpec.describe HTTParty do
       expect(@klass.headers).to eq(init_headers)
     end
 
+    it "should be able to accept block as header value" do
+      init_headers = {'foo' => lambda {'bar'}}
+      @klass.headers init_headers
+
+      stub_request(:get, "http://example.com/").with(headers: {'foo' => 'bar', 'baz' => 'spax'})
+
+      @klass.get('http://example.com/', headers: {'baz' => -> {'spax'}})
+      expect(@klass.headers).to eq(init_headers)
+    end
+
     it "uses the class headers when sending a request" do
-      expect_headers(foo: 'bar')
+      expect_headers('foo' => 'bar')
       @klass.headers(foo: 'bar')
       @klass.get('')
     end
 
     it "merges class headers with request headers" do
-      expect_headers(baz: 'spax', foo: 'bar')
+      expect_headers('baz' => 'spax', 'foo' => 'bar')
       @klass.headers(foo: 'bar')
       @klass.get('', headers: {baz: 'spax'})
     end
 
     it 'overrides class headers with request headers' do
-      expect_headers(baz: 'spax', foo: 'baz')
+      expect_headers('baz' => 'spax', 'foo' => 'baz')
       @klass.headers(foo: 'bar')
       @klass.get('', headers: {baz: 'spax', foo: 'baz'})
     end
 
     context "with cookies" do
       it 'utilizes the class-level cookies' do
-        expect_headers(foo: 'bar', 'cookie' => 'type=snickerdoodle')
+        expect_headers('foo' => 'bar', 'cookie' => 'type=snickerdoodle')
         @klass.headers(foo: 'bar')
         @klass.cookies(type: 'snickerdoodle')
         @klass.get('')
       end
 
       it 'adds cookies to the headers' do
-        expect_headers(foo: 'bar', 'cookie' => 'type=snickerdoodle')
+        expect_headers('foo' => 'bar', 'cookie' => 'type=snickerdoodle')
         @klass.headers(foo: 'bar')
         @klass.get('', cookies: {type: 'snickerdoodle'})
       end
@@ -184,7 +194,7 @@ RSpec.describe HTTParty do
       end
 
       it 'adds optional cookies to the optional headers' do
-        expect_headers(baz: 'spax', 'cookie' => 'type=snickerdoodle')
+        expect_headers('baz' => 'spax', 'cookie' => 'type=snickerdoodle')
         @klass.get('', cookies: {type: 'snickerdoodle'}, headers: {baz: 'spax'})
       end
     end
@@ -205,6 +215,21 @@ RSpec.describe HTTParty do
         @klass.post('http://example.com', body: { file: File.open('spec/fixtures/tiny.gif')})
       end
     end
+
+    context 'when headers passed as symbols' do
+      it 'converts them to string' do
+        expect_headers('foo' => 'application/json', 'bar' => 'example')
+        headers = { foo: 'application/json', bar: 'example' }
+        @klass.post('http://example.com', headers: headers)
+      end
+
+      it 'converts default headers to string' do
+        expect_headers('foo' => 'application/json', 'bar' => 'example')
+
+        @klass.headers(foo: 'application/json')
+        @klass.post('http://example.com', headers: { bar: 'example' })
+      end
+    end
   end
 
   describe "cookies" do
@@ -304,6 +329,10 @@ RSpec.describe HTTParty do
       @klass.default_timeout 0.5
       expect(@klass.default_options[:timeout]).to eq(0.5)
     end
+
+    it "should raise an exception if unsupported type provided" do
+      expect { @klass.default_timeout "0.5" }.to raise_error ArgumentError
+    end
   end
 
   describe "debug_output" do
@@ -783,8 +812,8 @@ RSpec.describe HTTParty do
 
   describe "#get" do
     it "should be able to get html" do
-      stub_http_response_with('google.html')
-      expect(HTTParty.get('http://www.google.com').parsed_response).to eq(file_fixture('google.html'))
+      stub_http_response_with('example.html')
+      expect(HTTParty.get('http://www.example.com').parsed_response).to eq(file_fixture('example.html'))
     end
 
     it "should be able to get chunked html" do
@@ -806,10 +835,28 @@ RSpec.describe HTTParty do
       expect(
         HTTParty.get('http://www.google.com', options) do |fragment|
           expect(chunks).to include(fragment)
+          expect(fragment.code).to eql 200
+          expect(fragment.http_response).to be
         end.parsed_response
       ).to eq(nil)
     end
 
+    context 'when streaming body' do
+      let(:chunk) { 'Content'.force_encoding('ascii-8bit') }
+      let(:options) { { stream_body: true } }
+      before do
+        stub_chunked_http_response_with([chunk], options) do |response|
+          allow(response).to receive(:[]).with('content-type').and_return('text/plain; charset = "utf-8"')
+        end
+      end
+
+      specify do
+        HTTParty.get('http://www.google.com', options) do |fragment|
+          expect(fragment.encoding).to eq(Encoding.find("UTF-8"))
+        end
+      end
+    end
+
     it "should be able parse response type json automatically" do
       stub_http_response_with('twitter.json')
       tweets = HTTParty.get('http://twitter.com/statuses/public_timeline.json')
@@ -865,16 +912,16 @@ RSpec.describe HTTParty do
     end
 
     it "should accept http URIs" do
-      stub_http_response_with('google.html')
+      stub_http_response_with('example.html')
       expect do
-        HTTParty.get('http://google.com')
+        HTTParty.get('http://example.com')
       end.not_to raise_error
     end
 
     it "should accept https URIs" do
-      stub_http_response_with('google.html')
+      stub_http_response_with('example.html')
       expect do
-        HTTParty.get('https://google.com')
+        HTTParty.get('https://example.com')
       end.not_to raise_error
     end
 
